Exception: CDC::Parallel::TimeoutError

Inherits:
Error
  • Object
show all
Defined in:
lib/cdc/parallel/errors.rb

Overview

Raised when a pool does not receive worker results before the configured timeout.

Timeout failures are normally returned inside ‘CDC::Core::ProcessorResult` failure objects rather than raised directly to the caller during result collection.