Exception: CDC::Parallel::Error
- Inherits:
-
StandardError
- Object
- StandardError
- CDC::Parallel::Error
- Defined in:
- lib/cdc/parallel/errors.rb
Overview
Base error for all cdc-parallel-specific failures.
Rescue this class when callers want to handle any failure raised directly by the parallel runtime layer.
Direct Known Subclasses
ProcessorExecutionError, ShutdownError, TimeoutError, UnsafeProcessorError, UnsupportedWorkItemError